import datetime

from ..utils import find_next_episode
from ..utils import numeric_ranges
from ..utils import numeric_range
from ..format import short_episode
from ..format import format_airdate
from ..format import format_days


def episode_key(now):
    def __episode_key(item):
        season, episode, airdate = item

        if episode is None or airdate is None:
            return 0

        return -((now - airdate).days)

    return __episode_key


def action(ns):
    before_color = ns.t.green
    inside_color = ns.t.yellow
    outside_color = ns.t.red
    all_seen_color = ns.t.magenta

    before, after = ns.range

    # prevent excessive time fetching.
    now = datetime.datetime.now()

    episode_sort_key = episode_key(now)

    next_episodes = list()
    all_seen = list()

    print ns.t.bold_magenta(u"Next episodes to watch out for")

    if not ns.all:
        print u"Airing within {0} and within {1}".format(
            before_color(format_days(before)),
            inside_color(format_days(after)))
    else:
        print (u"Airing within {0}, within {1}, "
               "outside {2} and {3}").format(
                   before_color(format_days(before)),
                   inside_color(format_days(after)),
                   outside_color(format_days(after)),
                   all_seen_color("never")
               )

    print u""

    for series in ns.series.list_series():
        episodes = ns.series.get_episodes(series)

        if episodes is None:
            print ns.t.bold_red(u"episodes not synced: {0}".format(
                series['series_name']))
            continue

        result = find_next_episode(
            episodes, ns.series.is_episode_watched,
            ignored_seasons=ns.ignore)

        if result is None:
            all_seen.append(series)
            continue

        next_episode, next_airdate = result
        next_episodes.append((series, next_episode, next_airdate))

    next_episodes = sorted(next_episodes, key=episode_sort_key)

    for series, episode, airdate in next_episodes:
        delta_days = abs((now - airdate).days)

        if after is not None and delta_days > after:
            if not ns.all:
                continue
            else:
                color = outside_color
        elif before is None or delta_days < before:
            color = before_color
        else:
            color = inside_color

        print color(u"{2} - {0} {1}".format(
            series['series_name'],
            short_episode(episode),
            format_airdate(episode['first_aired'], now=now)))

    if ns.all:
        for series in all_seen:
            print all_seen_color(u"never - {0}".format(
                series['series_name']))
            continue

    return 0


def setup(parser):
    parser.add_argument(
        '--ignore',
        '-i',
        metavar="<ranges>",
        help="Specify a set of seasons to ignore, defaults to '0'",
        default=set([0]),
        type=numeric_ranges,
    )

    parser.add_argument(
        '--all',
        '-a',
        help=("Display all series episodes, regardless of when they are "
              "airing."),
        default=False,
        action='store_const',
        const=True,
    )

    parser.add_argument(
        '--range',
        help=("Specify how many days ago are relevant, these will show up in "
              "a different color, default: 30"),
        default=(30, 90),
        type=numeric_range,
    )

    parser.set_defaults(action=action)



import datetime

from ..utils import has_aired_filter
from ..format import format_episodes_count
from ..format import format_episodes_count_legend


def series_name_key(s):
    return s['series_name']


def action(ns):
    print ns.t.bold_magenta(u"Series you are currently watching")
    print u""

    now = datetime.datetime.now()

    has_aired = has_aired_filter(now)

    series_dao = ns.series

    series_list = ns.series.list_series()

    episodes_legend = format_episodes_count_legend(ns.t)

    if series_list:
        for series in sorted(series_list, key=series_name_key):
            episodes = ns.series.get_episodes(series)

            episodes_count = None

            if episodes is not None:
                episodes_count, _ = format_episodes_count(
                    ns.t, series_dao, has_aired, episodes)

            print ns.t.bold_cyan(
                u"{0[series_name]} (id: {0[id]})".format(series))

            if episodes_count:
                print ns.t.cyan(
                    u"  Episodes ({0}): {1}".format(
                        episodes_legend, episodes_count))
    else:
        print ns.t.bold_red("You are not watching any series")

    return 0


def setup(parser):
    parser.set_defaults(action=action)



import unittest
import mock

from ontv.database import DictDB


class TestDatabaseDictDB(unittest.TestCase):
    def test_setitem(self):
        key = object()
        value = object()
        storage = mock.Mock()
        d = DictDB(storage, {})
        d[key] = value
        storage.setitem.assert_called_with(key, value)

    def test_delitem(self):
        key = object()
        value = object()
        storage = mock.Mock()
        d = DictDB(storage, {key: value})
        del d[key]
        storage.delitem.assert_called_with(key)

    def test_pop(self):
        key = object()
        value = object()
        storage = mock.Mock()
        d = DictDB(storage, {key: value})
        self.assertEquals(value, d.pop(key))
        storage.delitem.assert_called_with(key)

    def test_clear(self):
        key = object()
        value = object()
        storage = mock.Mock()
        d = DictDB(storage, {key: value})
        d.clear()
        self.assertTrue(storage.clear.called)

    def test_popitem(self):
        key = object()
        value = object()
        storage = mock.Mock()
        d = DictDB(storage, {key: value})
        self.assertEquals((key, value), d.popitem())
        storage.delitem.assert_called_with(key)

    def test_list_append(self):
        key = object()
        value = object()
        storage = mock.Mock()
        d = DictDB(storage, {key: []})
        d.list_append(key, value)
        storage.setitem.assert_called_with(key, [value])

    def test_list_remove(self):
        key = object()
        value = object()
        storage = mock.Mock()
        d = DictDB(storage, {key: [value]})
        d.list_remove(key, value)
        storage.setitem.assert_called_with(key, [])

    def test_compact(self):
        key = object()
        value = object()
        storage = mock.Mock()
        stats = storage.compact.return_value
        d = DictDB(storage, {key: value})
        self.assertEquals(stats, d.compact())
        storage.compact.assert_called_with([(key, value)])

    def test_db_size(self):
        storage = mock.Mock()
        size = storage.db_size.return_value
        d = DictDB(storage, {})
        self.assertEquals(size, d.db_size())



import contextlib
import json
import os
import collections


DictStorageStats = collections.namedtuple(
    "DictStorageStats",
    ['clears', 'deletions', 'noops'])


class Block(object):
    def load_value(self, v):
        raise NotImplementedError("load_value")

    def dump_value(self, v):
        raise NotImplementedError("dump_value")

    def load_key(self, key):
        return key

    def dump_key(self, key):
        return key

    def dump_header(self, fd):
        raise NotImplementedError("dump_header")

    def load_header(self, fd):
        raise NotImplementedError("load_header")

    def write_header(self, fd):
        raise NotImplementedError("write_header")

    def load_item(self, i):
        raise NotImplementedError("load_item")

    def write_entry(self, fd, op, key, value):
        raise NotImplementedError("write_entry")

    def load_entry(self, line):
        raise NotImplementedError("load_entry")


class JSONValueBlock(object):
    def load_value(self, v):
        if v == "":
            return None

        try:
            return json.loads(v)
        except:
            return None

    def dump_value(self, v):
        if v is None:
            return ""

        try:
            return json.dumps(v)
        except:
            return ""


class StandardHeader(object):
    def dump_header(self, d):
        return " ".join("=".join(i) for i in d.items())

    def load_header(self, s):
        if not s:
            return None

        return dict(s.split("=", 1) for s in s.split(" "))


class V1Block(JSONValueBlock, StandardHeader, Block):
    SPACE = " "
    DELIM = "\n"
    VERSION = '1.0'

    def __init__(self, encoding='utf-8'):
        self._encoding = encoding

    def header(self):
        return {'version': self.VERSION}

    def load_item(self, i):
        key, value = i
        return (self.load_key(key), self.load_value(value))

    def write_entry(self, fd, op, key, value):
        key = self.dump_key(key).encode(self._encoding)
        value = self.dump_value(value).encode(self._encoding)
        key_length = str(len(key))
        entry = self.SPACE.join((op, key_length, key, value)) + self.DELIM
        fd.write(entry)

    def load_entry(self, line):
        op, key_length, b = line.split(self.SPACE, 2)
        key_length = int(key_length)
        key = b[:key_length].decode(self._encoding)
        value = b[key_length + 1:-1].decode(self._encoding)
        return op, key, value

    def write_header(self, fd):
        fd.write(self.dump_header(self.header()) + self.DELIM)


class BlockYielder(object):
    def __init__(self, block_format, fd):
        self._block = block_format
        self._fd = fd

    def readblocks(self):
        if self._fd is None:
            raise Exception("file not open")

        try:
            for line in self._fd:
                op, key, value = self._block.load_entry(line)
                key = self._block.load_key(key)
                yield op, key, value
        finally:
            self._fd.close()
            self._fd = None

    def load_item(self, i):
        return self._block.load_item(i)


class FilesystemDriver(object):
    block_formats = {
        V1Block.VERSION: V1Block,
    }

    default_version = V1Block.VERSION

    def __init__(self, path, driver_open=open, encoding='utf-8'):
        self._path = path
        self._compact_path = os.path.join(
            os.path.dirname(path), u".{0}.compact".format(
                os.path.basename(path)))
        self._driver_open = driver_open
        self._encoding = encoding
        self._block = V1Block(encoding=self._encoding)

    def compact(self, entries):
        with self._driver_open(self._compact_path, 'w') as fd:
            self._block.write_header(fd)

            for op, key, value in entries:
                self._block.write_entry(fd, op, key, value)

        os.rename(self._compact_path, self._path)

    def appendlog(self, log):
        if not log:
            return

        with open(self._path, 'a') as fd:
            if fd.tell() == 0:
                self._block.write_header(fd)

            for op, key, value in log:
                self._block.write_entry(fd, op, key, value)

    def createyielder(self):
        fd = self._driver_open(self._path, 'a+')

        header_line = fd.readline()

        if header_line:
            header = self._block.load_header(header_line.strip())
        else:
            header = dict(version=self.default_version)

        # setup temporary block format for reading.
        # this is useful for migrating old database formats behind the scenes.
        # The next time the database is compacted, the block format will
        # change.
        block_format = self.block_formats.get(header['version'])

        if block_format is None:
            raise Exception(
                "unknown block format version: {0}".format(header['version']))

        block_format = block_format(encoding=self._encoding)

        return BlockYielder(block_format, fd)


class DictStorage(object):
    # add a single value.
    VALUE = "+"

    # delete a single value.
    DELETION = "-"

    # clear the entire data set.
    CLEAR = "X"

    def __init__(self, driver):
        self._driver = driver
        self._log = list()
        self._stats = None

    def read(self):
        self._stats, data = self._read_cache(self._driver.createyielder())
        return data

    def stats(self):
        return self._stats

    def _read_cache(self, yielder):
        data = dict()

        # generate statistics to decide if we should autocompact the database.
        clears = 0
        deletions = 0

        for op, key, value in yielder.readblocks():
            if op == self.DELETION:
                del data[key]
                deletions += 1
                continue

            if op == self.CLEAR:
                data.clear()
                clears += 1
                continue

            if op == self.VALUE:
                data[key] = value
                continue

            raise Exception("unknown operation '{0}'".format(op))

        data = dict(yielder.load_item(i) for i in data.items())

        stats = DictStorageStats(
            clears=clears,
            deletions=deletions,
            noops=deletions + clears)

        return stats, data

    def commit(self):
        self._driver.appendlog(self._log)
        self._log = list()

    def setitem(self, key, value):
        self._log.append((self.VALUE, key, value))

    def delitem(self, key):
        self._log.append((self.DELETION, key, None))

    def clear(self, key):
        self._log.append((self.CLEAR, "", None))

    def compact(self, items):
        generator = ((self.VALUE, key, value) for key, value in items)

        self._driver.compact(generator)

        stats = self._stats

        self._stats = DictStorageStats(
            clears=0, deletions=0, noops=0)

        return stats

    def db_size(self):
        return self._driver.db_size()


class DictDB(dict):
    def __init__(self, storage, cache):
        self._storage = storage
        dict.__init__(self, cache)

    def __setitem__(self, key, data):
        dict.__setitem__(self, key, data)
        self._storage.setitem(key, data)

    def __delitem__(self, key):
        dict.__delitem__(self, key)
        self._storage.delitem(key)

    def pop(self, key, *args, **kw):
        value = dict.pop(self, key, *args, **kw)
        self._storage.delitem(key)
        return value

    def clear(self):
        dict.clear(self)
        self._storage.clear()

    def setdefault(self, **args):
        raise NotImplementedError("setdefault")

    def update(self, **args):
        raise NotImplementedError("update")

    def popitem(self, **args):
        key, value = dict.popitem(self)
        self._storage.delitem(key)
        return key, value

    def list_append(self, key, data):
        array = self.get(key, [])
        array.append(data)
        self.__setitem__(key, array)

    def list_remove(self, key, data):
        array = self.get(key, [])
        array.remove(data)
        self.__setitem__(key, array)

    def compact(self):
        return self._storage.compact(self.items())

    def db_size(self):
        return self._storage.db_size()


class SetDB(set):
    def __init__(self, storage, cache):
        self._storage = storage
        set.__init__(self, cache.keys())

    def add(self, key):
        if key in self:
            return

        set.add(self, key)
        self._storage.setitem(key, None)

    def remove(self, key):
        set.remove(self, key)
        self._storage.delitem(key)

    def pop(self):
        key = set.pop(self)
        self._storage.delitem(key)
        return key

    def compact(self):
        return self._storage.compact((v, None) for v in self)

    def db_size(self):
        return self._storage.db_size()


@contextlib.contextmanager
def open_database(
    path,
    compaction_limit=1000,
    impl=DictDB,
    driver_open=open,
    driver=FilesystemDriver,
):
    driver_instance = driver(path, driver_open=driver_open)

    storage = DictStorage(driver_instance)

    cache = storage.read()

    stats = storage.stats()

    db = impl(storage, cache)

    # noops means non-operations, basically operations that does not contribute
    # to the final structure of the data.
    #
    # if we hit a limit here, we should cleanup, otherwise it's a waste of
    # space.
    if stats.noops > compaction_limit:
        db.compact()

    yield db
    storage.commit()



from ..format import print_series


def action(ns):
    print ns.t.bold_magenta(u"Results from searching online database")
    print u""

    if not ns.api.is_authenticated():
        print ns.t.bold_red(u"Not authenticated")
        return 1

    series = ns.api.getseries(ns.query)

    if not series:
        print ns.t.bold_red("No series matching '{0}'".format(ns.query))
        return 1

    if ns.limit:
        series = series[:ns.limit]

    for s in series:
        print_series(ns.t, s)


def setup(parser):
    parser.add_argument(
        "query",
        metavar="<query>",
    )

    parser.add_argument(
        "--limit",
        metavar="<number>",
        help="Limit the amount of search results displayed, default: 5.",
        type=int,
        default=5)

    parser.set_defaults(action=action)



import unittest
import mock

from ontv.database import DictStorage


class TestDictStorage(unittest.TestCase):
    @mock.patch.object(DictStorage, '_read_cache')
    def test_read(self, read_cache):
        stats = object()
        ref_data = object()

        read_cache.return_value = (stats, ref_data)

        driver = mock.Mock()
        storage = DictStorage(driver)

        # just initialized storage has no stats.
        self.assertEquals(None, storage.stats())
        self.assertEquals(ref_data, storage.read())
        self.assertEquals(stats, storage.stats())

    def test_commit(self):
        ref_log = []

        driver = mock.Mock()
        storage = DictStorage(driver)
        storage.commit()

        driver.appendlog.assert_called_with([])
        self.assertEquals(ref_log, storage._log)

    def test_setitem(self):
        key = object()
        value = object()

        ref_log = [
            (DictStorage.VALUE, key, value)
        ]

        driver = mock.Mock()
        storage = DictStorage(driver)
        storage.setitem(key, value)

        self.assertEquals(ref_log, storage._log)

    def test_delitem(self):
        key = object()

        ref_log = [
            (DictStorage.DELETION, key, None)
        ]

        driver = mock.Mock()
        storage = DictStorage(driver)
        storage.delitem(key)

        self.assertEquals(ref_log, storage._log)

    def test_clear(self):
        key = object()

        ref_log = [
            (DictStorage.CLEAR, "", None)
        ]

        driver = mock.Mock()
        storage = DictStorage(driver)
        storage.clear(key)

        self.assertEquals(ref_log, storage._log)



import textwrap
import datetime
from dateutil import relativedelta

from .utils import has_aired_filter


def short_episode(episode):
    return u"S{0:02}E{1:02}".format(
        episode['season_number'], episode['episode_number'])


def readable_timedelta(now, then, next_day, suffix="", prefix=""):
    delta = relativedelta.relativedelta(now, then)

    years = abs(delta.years)
    months = abs(delta.months)
    days = abs(delta.days)

    if years == 0 and months == 0 and days == 0:
        return "today"

    if years == 0 and months == 0 and days == 1:
        return next_day

    date = list()

    if years != 0:
        if years == 1:
            date.append("1 year")
        else:
            date.append("{0} years".format(years))

    if years < 2 and months != 0:
        if months == 1:
            date.append("1 month")
        else:
            date.append("{0} months".format(months))

    if months == 0:
        if days == 1:
            date.append("1 day")
        else:
            date.append("{0} days".format(days))

    return prefix + ", ".join(date) + suffix


def floor_datetime(dt):
    return datetime.datetime(
        year=dt.year,
        month=dt.month,
        day=dt.day,
    )


def format_airdate(aired, now=None):
    if aired is None:
        return "n/a"

    try:
        then = datetime.datetime.strptime(aired, "%Y-%m-%d")
    except:
        return "<invalid %Y-%m-%d>"

    if now is None:
        now = datetime.datetime.now()

    now = floor_datetime(now)

    if then <= now:
        return readable_timedelta(
            now, then, "yesterday", suffix=" ago")

    return readable_timedelta(
        now, then, "tomorrow", prefix="in ")


def format_days(days):
    if days == 1:
        return "1 day"

    return "{0} days".format(days)


def print_wrapped(text, indent=""):
    wrapper = textwrap.TextWrapper()
    wrapper.initial_indent = indent
    wrapper.subsequent_indent = indent

    for line in wrapper.wrap(text):
        print line


def print_title(term, title):
    print term.bold_cyan(title)


def print_episode(
    term, series_dao, episode,
    short_version=True,
    indent="",
):
    color = term.white

    if series_dao.is_episode_watched(episode):
        color = term.bold_blue

    if short_version:
        print color(
            u"{0}{1:02} '{2}' {3} ({4})".format(
                indent, episode['episode_number'],
                episode['episode_name'],
                format_airdate(episode['first_aired']),
                episode['first_aired']))
    else:
        print color(
            u"{0}{1:02} {2}".format(
                indent, episode['episode_number'],
                episode['episode_name']))

    if short_version:
        return

    print term.cyan(u"{0}Air date: {1} ({2})".format(
        indent + "  ",
        format_airdate(episode['first_aired']),
        episode['first_aired']))

    if episode['overview']:
        print_wrapped(episode['overview'], indent=indent + u"  ")

    if 'guest_stars' in episode:
        print term.cyan(u"{0}Guest stars:".format(indent + u"  "))
        print_wrapped(format_compact_list(episode['guest_stars']),
                      indent=indent + u"    ")


def print_season(
        term, series_dao,
        series,
        season_number,
        episodes=None,
        indent=""):

    now = datetime.datetime.now()

    has_aired = has_aired_filter(now)

    episodes_legend = format_episodes_count_legend(term)

    all_episodes = series_dao.get_season_episodes(series, season_number)

    episodes_count, stats = format_episodes_count(
        term, series_dao, has_aired, all_episodes)

    color = term.white

    seen, aired, all = stats

    if seen == aired or seen == all:
        color = term.bold_green
    elif seen > 0:
        color = term.bold_yellow
    else:
        color = term.bold_red

    print u"{0}{c}Season {1}{t.normal} ({2}): {3}".format(
        indent, season_number, episodes_legend, episodes_count,
        c=color, t=term)

    if not episodes:
        return

    for episode in episodes:
        print_episode(
            term, series_dao, episode,
            short_version=False,
            indent=indent + "  ")


def print_list(items, item_format=u"- {0}", indent=""):
    if items is None:
        print u"{0}(empty)".format(indent)
        return

    for item in items:
        print u"{0}{1}".format(indent, item_format.format(item))


def format_compact_list(items, item_format=u"{0}"):
    if items is None:
        return u"(empty)"

    return u", ".join(map(item_format.format, items))


def print_series(
    term, series,
    seasons=None,
    ignored_seasons=set(),
    series_dao=None,
    indent=u"",
):
    print_title(term, u"{0} (id: {1})".format(
        series['series_name'], series['series_id']))

    if 'first_aired' in series:
        print term.cyan(u"Air date: {0} ({1})".format(
            format_airdate(series['first_aired']),
            series['first_aired']))

    if 'overview' in series:
        if series['overview']:
            print_wrapped(series['overview'], indent="  ")

    if not seasons:
        return

    if 'actors' in series:
        print term.cyan(u"{0}Actors:".format(indent))
        print_wrapped(format_compact_list(series['actors']),
                      indent=indent + u"  ")

    print term.cyan(u"Seasons")

    for season_number, season_episodes in sorted(seasons.items()):
        if season_number in ignored_seasons:
            continue

        if len(seasons) != 1:
            season_episodes = None

        print_season(
            term, series_dao, series, season_number,
            episodes=season_episodes,
            indent="  ")


def format_episodes_count_legend(term):
    return (
        u"{t.green}{0}{t.normal}/"
        u"{t.yellow}{1}{t.normal}/"
        u"{t.red}{2}{t.normal}"
    ).format(
        "seen", "aired", "all",
        t=term)


def format_episodes_count(term, series_dao, has_aired, episodes):
    seen_episodes = len(filter(
        series_dao.is_episode_watched, episodes))

    aired_episodes = len(filter(has_aired, episodes))

    all_episodes = len(episodes)

    stats = (seen_episodes, aired_episodes, all_episodes)

    return (
        u"{t.green}{0}{t.normal}/"
        u"{t.yellow}{1}{t.normal}/"
        u"{t.red}{2}{t.normal}"
    ).format(*stats, t=term), stats



import requests
from lxml import etree

DEFAULT_URL = "http://thetvdb.com"


def text_value(element_type):
    def __inner(c):
        if not c.text:
            return None
        return element_type(c.text)

    return __inner


def text_list(element_type, separator="|"):
    """
    Parse a list contained withing a text field.
    """

    def __inner(c):
        if not c.text:
            return None

        results = list()

        for item in c.text.split(separator):
            item = item.strip()

            if not item:
                continue

            results.append(element_type(item))

        return results

    return __inner


class TheTVDBApi(object):
    MIRROR = {
        "id": ("id", text_value(int)),
        "mirrorpath": ("mirrorpath", text_value(unicode)),
        "typemask": ("typemask", text_value(int)),
    }

    LANGUAGE = {
        "id": ("id", text_value(int)),
        "abbreviation": ("abbreviation", text_value(unicode)),
        "name": ("name", text_value(unicode)),
    }

    SERIES = {
        "id": ("id", text_value(int)),
        "seriesid": ("series_id", text_value(int)),
        "language": ("language", text_value(unicode)),
        "SeriesName": ("series_name", text_value(unicode)),
        "Overview": ("overview", text_value(unicode)),
        "FirstAired": ("first_aired", text_value(unicode)),
    }

    BASE_SERIES = {
        "id": ("id", text_value(int)),
        "Actors": ("actors", text_list(unicode)),
        "Airs_DayOfWeek": ("airs_day_of_week", text_value(unicode)),
        "Airs_Time": ("airs_time", text_value(unicode)),
        "FirstAired": ("first_aired", text_value(unicode)),
        "Genre": ("genre", text_list(unicode)),
        "IMDB_ID": ("imdb_id", text_value(unicode)),
        "Language": ("language", text_value(unicode)),
        "Network": ("network", text_value(unicode)),
        "NetworkID": ("network_id", text_value(int)),
        "Overview": ("overview", text_value(unicode)),
        "Rating": ("rating", text_value(float)),
        "Runtime": ("runtime", text_value(int)),
        "SeriesID": ("series_id", text_value(int)),
        "SeriesName": ("series_name", text_value(unicode)),
        "Status": ("status", text_value(unicode)),
        "added": ("added", text_value(unicode)),
        "addedBy": ("added_by", text_value(unicode)),
        "lastupdated": ("lastupdated", text_value(int)),
    }

    EPISODE = {
        "id": ("id", text_value(int)),
        "EpisodeNumber": ("episode_number", text_value(int)),
        "EpisodeName": ("episode_name", text_value(unicode)),
        "SeasonNumber": ("season_number", text_value(int)),
        "Overview": ("overview", text_value(unicode)),
        "FirstAired": ("first_aired", text_value(unicode)),
        "GuestStars": ("guest_stars", text_list(unicode)),
        "Writer": ("writer", text_list(unicode)),
        "seasonid": ("season_id", text_value(int)),
        "seriesid": ("series_id", text_value(int)),
        "lastupdated": ("lastupdated", text_value(int)),
    }

    def __init__(self, api_key, base_url=None):
        if base_url is None:
            base_url = DEFAULT_URL

        if api_key:
            self.api_url = "{0}/api/{1}".format(base_url, api_key)
        else:
            self.api_url = None

        self.open_api_url = "{0}/api".format(base_url)
        self.session = requests.session()

    def _request(self, method, base_url, path, **kw):
        if base_url is None:
            raise Exception("Invalid base url")

        url = "{0}/{1}".format(base_url, path)

        r = self.session.get(url, **kw)

        if not r.ok:
            raise Exception("request failed: {0} ({1})".format(
                url, r.status_code))

        return etree.XML(r.content)

    def is_authenticated(self):
        return self.api_url is not None

    def _assert_api_key(self):
        if self.api_url is None:
            raise Exception("No API key specified")

    def get(self, path, **kw):
        self._assert_api_key()
        return self._request('GET', self.api_url, path, **kw)

    def open_get(self, path, **kw):
        return self._request('GET', self.open_api_url, path, **kw)

    def _parse_element(self, element, types):
        result = dict()

        for key, _ in types.values():
            result[key] = None

        for c in element:
            element_specifier = types.get(c.tag)

            if element_specifier is None:
                continue

            target_key, element_type = element_specifier

            try:
                result[target_key] = element_type(c)
            except Exception as e:
                raise Exception(
                    "Failed to parse field '{0}': {1}".format(
                        c.tag, str(e)))

        return result

    def mirrors(self):
        document = self.get('mirrors.xml')
        return [self._parse_element(child, self.MIRROR)
                for child in document.iter("Mirror")]

    def languages(self):
        document = self.get('languages.xml')
        return [self._parse_element(child, self.LANGUAGE)
                for child in document.iter("Language")]

    def getseries(self, seriesname):
        document = self.open_get(
            'GetSeries.php',
            params={"seriesname": seriesname})

        return [self._parse_element(child, self.SERIES)
                for child in document.iter("Series")]

    def series(self, series_id, language):
        document = self.get(
            'series/{0}/{1}.xml'.format(series_id, language))

        series = document.find("Series")

        if not len(series):
            return None

        return self._parse_element(series, self.BASE_SERIES)

    def series_all(self, series_id, language):
        document = self.get(
            'series/{0}/all/{1}.xml'.format(series_id, language))

        series = document.find("Series")
        episodes = document.findall("Episode")

        if len(series):
            series = self._parse_element(series, self.BASE_SERIES)
        else:
            series = None

        episodes = [self._parse_element(episode, self.EPISODE)
                    for episode in episodes]

        return series, episodes



from ..utils import write_yaml


def action(ns):
    print ns.t.bold_magenta(u"Synchronizing local database")
    print u""

    if not ns.api.is_authenticated():
        print ns.t.bold_red(
            "Cannot fully synchronize, api is not authenticated with a key")
        print ns.t.bold_red(
            "Set a value to the 'api_key' option in {0}".format(
                ns.config_path))
        return 0

    if not ns.languages:
        with open(ns.languages_path, 'w') as fp:
            write_yaml(fp, ns.api.languages())

    if not ns.mirrors:
        print ns.t.bold_green("Downloading mirrors to {0}".format(
            ns.mirrors_path))

        with open(ns.mirrors_path, 'w') as fp:
            write_yaml(fp, ns.api.mirrors())

    for series in ns.series.list_series():
        print ns.t.bold_cyan(
            u"syncing: {0}".format(series['series_name']))

        series, episodes = ns.api.series_all(
            series['id'], ns.language)

        ns.series.set_episodes(series, episodes)

    return 0


def setup(parser):
    parser.set_defaults(action=action)



from ..utils import with_resource
from ..utils import local_series_finder


@with_resource(local_series_finder)
def action(ns, series):
    ns.series.remove(series)
    print ns.t.green("removed: {0}".format(series['series_name']))
    return 0


def setup(parser):
    parser.add_argument(
        "series_query",
        metavar="<id|name>",
        help="The id of the series to remove",
    )

    parser.set_defaults(action=action)



from distutils.core import setup

VERSION = '0.6.6'


setup(
    name='ontv',
    version=VERSION,
    description="Your personal TV series manager",
    long_description="""
    ontv is a utility that helps you keep track of what you are looking at
    right now.

    It is based of data from thetvdb.com which requires an account and an api
    key to access, but contains information about most existing tv shows and
    their air date.

    My source of focus was to fast create a usable application that tracks the
    tv shows I am currently watching.
    """,
    author='John-John Tedro',
    author_email='johnjohn.tedro@gmail.com',
    url='http://github.com/udoprog/ontv',
    license='GPLv3',
    packages=[
        'ontv',
        'ontv.action'
    ],
    scripts=['bin/ontv'],
    install_requires=[
        'PyYAML>=3.10',
        'blessings>=1.5',
        'requests>=1.1.0',
        'python-dateutil>=2.1',
    ]
)
